#ifndef __avr_mapping_h
#define __avr_mapping_h

class AVR8
{
public:
    typedef volatile unsigned char  Reg8;
    typedef volatile unsigned short Reg16;
    typedef volatile unsigned char  IOReg8;

    Reg8 r0;
    Reg8 r1;
    Reg8 r2;
    Reg8 r3;
    Reg8 r4;
    Reg8 r5;
    Reg8 r6;
    Reg8 r7;
    Reg8 r8;
    Reg8 r9;
    Reg8 r10;
    Reg8 r11;
    Reg8 r12;
    Reg8 r13;
    Reg8 r14;
    Reg8 r15;
    Reg8 r16;
    Reg8 r17;
    Reg8 r18;
    Reg8 r19;
    Reg8 r20;
    Reg8 r21;
    Reg8 r22;
    Reg8 r23;
    Reg8 r24;
    Reg8 r25;
    union {
	struct{Reg8 r26; Reg8 r27;};
	Reg16 x;
    };
    union {
	struct{Reg8 r28; Reg8 r29;};
	Reg16 y;
    };
    union {
	struct{Reg8 r30; Reg8 r31;};
	Reg16 z;
    };
};

class AT90S: public AVR8
{
public:
    static const unsigned short RAM_SIZE   = 0x0200;

public:
    IOReg8 reserved_00;	// [0x20] Reserved
    IOReg8 reserved_01;	// [0x21] Reserved
    IOReg8 reserved_02;	// [0x22] Reserved
    IOReg8 reserved_03;	// [0x23] Reserved
    IOReg8 reserved_04;	// [0x24] Reserved
    IOReg8 reserved_05;	// [0x25] Reserved
    IOReg8 reserved_06;	// [0x26] Reserved
    IOReg8 reserved_07;	// [0x27] Reserved
    IOReg8 acsr;	// [0x28] Analog Comparator Control and Status Register
    IOReg8 ubrr;	// [0x29] UART Baud Rate Register
    IOReg8 ucr;		// [0x2A] UART Control Register
    IOReg8 usr;		// [0x2B] UART Status Register
    IOReg8 udr;		// [0x2C] UART I/O Data Register
    IOReg8 spcr;	// [0x2D] SPI Control Register
    IOReg8 spsr;	// [0x2E] SPI Status Register
    IOReg8 spdr;	// [0x2F] SPI I/O Data Register
    IOReg8 ind;		// [0x30] Input Pins, Port D
    IOReg8 ddrd;	// [0x31] Data Direction Register, Port D
    IOReg8 portd;	// [0x32] Data register, Port D
    IOReg8 inc;		// [0x33] Input Pins, Port C
    IOReg8 ddrc;	// [0x34] Data Direction Register, Port C
    IOReg8 portc;	// [0x35] Data register, Port C
    IOReg8 inb;		// [0x36] Input Pins, Port B
    IOReg8 ddrb;        // [0x37] Data Direction Register, Port B
    IOReg8 portb;       // [0x38] Data register, Port B
    IOReg8 ina;		// [0x39] Input Pins, Port A
    IOReg8 ddra;	// [0x3A] Data Direction Register, Port A (1->out, 0->in)
    IOReg8 porta;	// [0x3B] Data register, Port A
    IOReg8 eecr;	// [0x3C] EEPROM Control Register
    IOReg8 eedr;	// [0x3D] EEPROM Data Register
    IOReg8 eearl;	// [0x3E] EEPROM Address Register Low Byte
    IOReg8 eearh;	// [0x3F] EEPROM Address Register High Byte (AT90S8515)
    IOReg8 reserved_20; // [0x40] Reserved
    IOReg8 wstcr;	// [0x41] Watchdog Timer Control Register
    IOReg8 reserved_22;	// [0x42] Reserved
    IOReg8 reserved_23;	// [0x43] Reserved
    IOReg8 icr1l;	// [0x44] Timer/Counter 1 Input Capture Register Low Byte
    IOReg8 icr1h;	// [0x45] Timer/Counter 1 Input Capture Register High Byte
    IOReg8 reserved_26;	// [0x46] Reserved
    IOReg8 reserved_27;	// [0x47] Reserved
    IOReg8 ocr1bl;	// [0x48] Timer/Counter 1 Output Compare Register B Low Byte
    IOReg8 ocr1bh;	// [0x49] Timer/Counter 1 Output Compare Register B High Byte
    IOReg8 ocr1al;	// [0x4A] Timer/Counter 1 Output Compare Register A Low Byte
    IOReg8 ocr1ah;	// [0x4B] Timer/Counter 1 Output Compare Register A High Byte
    IOReg8 tcnt1l;	// [0x4C] Timer/COunter1 Low Byte
    IOReg8 tcnt1h;      // [0x4D] Timer/Counter1 High Byte
    IOReg8 tccr1b;	// [0x4E] Timer/Counter1 Control Register B
    IOReg8 tccr1a;	// [0x4F] Timer/Counter1 Control Register A
    IOReg8 reserved_30;	// [0x50] Reserved
    IOReg8 reserved_31;	// [0x51] Reserved
    IOReg8 tcnt0;	// [0x52] Timer/Counter0 (8-bit)
    IOReg8 tccr0;	// [0x53] Timer/Counter0 Control Register
    IOReg8 reserved_34;	// [0x54] Reserved
    IOReg8 mcucr;	// [0x55] MCU General Control Register
    IOReg8 reserved_36;	// [0x56] Reserved
    IOReg8 reserved_37;	// [0x57] Reserved
    IOReg8 tifr;	// [0x58] Timer/Counter Interrupt Flag Register
    IOReg8 timsk;	// [0x59] Timer/Counter Interrupt Mask Register
    IOReg8 gifr;	// [0x5A] General Interrupt Flag Register
    IOReg8 gimsk;	// [0x5B] General Interrupt Mask Register
    IOReg8 reserved_3c;	// [0x5C] Reserved
    IOReg8 spl;		// [0x5D] Stack Pointer Low
    IOReg8 sph;		// [0x5E] Stack Pointer High
    IOReg8 sreg;	// [0x5F] Status Register

    char ram[RAM_SIZE];
};

enum PORTS {
        DDRB  = 0x19,
        PORTB = 0x18,
        SREG  = 0x3F,
        GIMSK = 0x3B,
        TIMSK = 0x39,
        MCUCR = 0x35,
	TCCR0 = 0x33,
};

#endif
